/*****************************************************************************
	P3 Syntax Coloring Parser 
	by Sergey V. Mishchenko (seras@mail.ru)
	Version 1.02 build 04/12/2004
/*****************************************************************************/

// Elements
#define clOrange			33023
#define clBlood				204
#define clSoftBlue			12615680
#define clCobalt			15615044
#define clGrayPlus			10066329
#define clSpGreen			4227072

#define DEFAULT				0
#define XML_TAGSTART		1
#define XML_DEFSTART		2
#define XML_ENTITY			3
#define XML_COMMENT			4
#define XML_DEFEND			5
#define PARSER_VARIABLE		6
#define PARSER_VARIABLE2	7
#define PARSER_FUNCTION		8
#define P_COMMENT_LINE		9
#define P_COMMENT_BLOCK		10
#define PARSER_SERVICE		11
#define GLOBAL_BRACKET		12
#define PARSER_RESERVED		13
#define PARSER_RESULT		14
#define PARSER_GLOBAL		15
#define XML_TAGEND			16
#define XML_TAGNAME			17
#define ATTR_STRING			18
#define XML_NUMBER			19
#define LOGEXP_START		20
#define LOGEXP_END			21
#define LOGEXP_OPERATOR		22
#define LOGEXP_COMMENT		23
#define XML_COMSTART		24
#define XML_COMEND			25
#define P_DEFSTART			26
#define P_DEFEND			27
#define P_DEFNAME			28
#define P_DEFCOMMENT		29
#define P_DELIMITER			30
#define GLOBAL_NUMBER		31
#define FORMAT_STRING		32
#define XML_DEFNAME			33
//#define XML_DEFATTR			34
#define XML_DEFNUMBER		35
#define SYMBOL_CODE			36
#define XML_TAGCLOSE		37
#define LOGEXP_STRING		38
#define DPARAM_START		39
#define DPARAM_END			40
#define DPARAM_SEP			41
#define DEFCOM_START		42
#define DEFCOM_END			43
#define TAGNAME_MISC		44
#define NAMESPACE_SEP		45
#define TAGATTR_START		46
#define TAGATTR_END			47
#define ATTR_COM_LINE		48
#define DEFATTR_START		49
#define DEFATTR_END			50

// States
#define IN_DEFAULT			0
#define IN_TAG				1
#define IN_LOGEXP			2
#define IN_DEFINEM			3
#define IN_DEFXML			4
#define IN_DEFPARAM			5
#define IN_DEFCOM			6
#define IN_TAGCLOSE			7
#define IN_TAGATTR			8
#define IN_DEFATTR			9

// Common Expressions
#define _ALPHA				'[_A-Za-z0-9]'
#define _NOT_ALPHA			'[^_A-Za-z0-9]'
#define _BETA				'[_A-Za-z0-9\-\.\:]'
#define _GAMA				'[_A-Za-z0-9\.]'
#define _NOT_BETA			'[^_A-Za-z0-9\-\.\:]'
#define _ALL_CHARS			'[\x00-\xFF]'
#define _NO_CHARS			'[]'
#define _DONT_CARE			_ALL_CHARS
#define _XML_START			'[_A-Za-z\:\$\^]'
#define _ATTR_BEGIN			'[\x23-\xFF!]'
#define _ATTR_ALLOW			'[^\x00-\x22\$\^\"\)\(\{\}\]\[;]'
#define _ATTR_CLOSE			'[\x00-\x22\$\^\"\)\(\{\}\]\[;]'

#define DEFAULT_BACKGROUND	clWhite
#define DEFAULT_FOREGROUND	clBlack

%%language
Name			=	'Parser 3'
Case			=	__SENSITIVE
StartState		=	[IN_DEFAULT]
EditableStyles	=	('Default Text'         , DEFAULT),
					('XML Tags'             , XML_TAGNAME),
					('XML Instructions'     , XML_DEFNAME),
					('XML Entities'         , XML_ENTITY),
					('XML Comments'         , XML_COMSTART),
					('XML Attribute'        , TAGATTR_START),
					('P3 Variables'         , PARSER_VARIABLE),
					('P3 Methods'           , PARSER_FUNCTION),
					('P3 Comments'          , P_COMMENT_LINE),
					('P3 Method Definition'    , P_DEFSTART),
					('P3 Reserved, P3 Brackets'      , GLOBAL_BRACKET),
					('P3 Method Result'        , PARSER_RESULT),
					('P3 Expression Operator'  , LOGEXP_OPERATOR),
					('P3 Expression String'    , LOGEXP_STRING),
					('P3 Number'            , GLOBAL_NUMBER),
					('P3 Format String'     , FORMAT_STRING),
					('P3 Service'           , PARSER_SERVICE)
ExampleText		=	'# $Id: simple.p,v 1.02 2004/04/12 15:21:37 VPoupkine Exp $\n\ 
					\ \n\
					\@USE\n\
					\/../classes/power_ops.p\n\
					\\n\
					\@customMethod[messages;status] comments\n\
					\^if(def $messages && $status.int(0) != 0){\n\
					\	$result[^xdoc::create{<?xml version="1.0"?>\n\
					\		<document>\n\
					\		^messages.menu{\n\
					\			<message>\n\
					\				<id>^messages.id.format[%02d]</id>\n\
					\				<text>^untaint{$messages.body}</text>\n\
					\			</message>\n\
					\		}\n\
					\		</document>\n\
					\	]\n\
					\}'

%%words
'<'				_XML_START			XML_TAGSTART		[IN_DEFAULT]
'</'			_XML_START			XML_TAGCLOSE		[IN_DEFAULT]
'>'				_DONT_CARE			XML_TAGEND			[IN_TAG IN_TAGCLOSE]
'/>'			_DONT_CARE			XML_TAGEND			[IN_TAG]
'?>'			_DONT_CARE			XML_DEFEND			[IN_DEFXML]
'='				_DONT_CARE			XML_DEFNAME			[IN_DEFXML]
'='				_DONT_CARE			XML_TAGNAME			[IN_TAG]
'<?'			_XML_START			XML_DEFSTART		[IN_DEFAULT]
'<!--'			_DONT_CARE			XML_COMSTART		[IN_DEFAULT]
'\-\->'			_DONT_CARE			XML_COMEND			[IN_DEFAULT]
'\n@'			_ALPHA				P_DEFSTART			[IN_DEFAULT]
'\n'			_DONT_CARE			P_DEFEND			[IN_DEFINEM]
' '				_DONT_CARE			P_DEFCOMMENT		[IN_DEFCOM]
'['				'[^\n]'				DPARAM_START		[IN_DEFINEM]
'\n'			_DONT_CARE			DPARAM_END			[IN_DEFPARAM]
']['			_DONT_CARE			DPARAM_SEP			[IN_DEFPARAM]
'['				_DONT_CARE			DPARAM_SEP			[IN_DEFPARAM]
';'				_DONT_CARE			DPARAM_SEP			[IN_DEFPARAM]
']'				'[\n ]'				DEFCOM_START		[IN_DEFPARAM]
'\n'			_DONT_CARE			DEFCOM_END			[IN_DEFCOM]
':'				_BETA				NAMESPACE_SEP		[IN_TAG IN_TAGCLOSE IN_DEFXML]
'-'				_BETA				TAGNAME_MISC		[IN_TAG IN_TAGCLOSE IN_DEFXML]
'"'				_DONT_CARE			TAGATTR_START		[IN_TAG]
'"'				_DONT_CARE			DEFATTR_START		[IN_DEFXML]
'"'				_DONT_CARE			TAGATTR_END			[IN_TAGATTR]
'"'				_DONT_CARE			DEFATTR_END			[IN_DEFATTR]
'\n'			'[\#]'				ATTR_COM_LINE		[IN_TAGATTR IN_DEFATTR]
//
'^if('			_DONT_CARE			LOGEXP_START		[IN_DEFAULT]
'^while('		_DONT_CARE			LOGEXP_START		[IN_DEFAULT]
'){'			_DONT_CARE			LOGEXP_END			[IN_LOGEXP]
' '				'\#'				LOGEXP_COMMENT		[IN_LOGEXP]
' is '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'def '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'in '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'-f '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'-d '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'='				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'!'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'|'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'>'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'<'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'&'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'%'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'+'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
'-'				_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' ne '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' eq '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' lt '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' gt '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' le '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
' ge '			_DONT_CARE			LOGEXP_OPERATOR		[IN_LOGEXP]
//
'$'				_GAMA				PARSER_VARIABLE		[IN_DEFAULT]
'${'			_GAMA				PARSER_VARIABLE2	[IN_DEFAULT]
'^'				_ALPHA				PARSER_FUNCTION		[IN_DEFAULT]
'^rem{'			_DONT_CARE			P_COMMENT_BLOCK		[IN_DEFAULT]
'\n@CLASS\n'	_DONT_CARE			PARSER_SERVICE		[IN_DEFAULT]
'\n@BASE\n'		_DONT_CARE			PARSER_SERVICE		[IN_DEFAULT]
'\n@USE\n'		_DONT_CARE			PARSER_SERVICE		[IN_DEFAULT]
'^MAIN'			':'					PARSER_GLOBAL		[IN_DEFAULT]
'^BASE'			':'					PARSER_GLOBAL		[IN_DEFAULT]
'$result'		_NOT_BETA			PARSER_RESULT		[IN_DEFAULT]
//
'('				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
')'				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
'['				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
']'				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
'{'				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
'}'				_DONT_CARE			GLOBAL_BRACKET		[IN_DEFAULT]
';'				_DONT_CARE			P_DELIMITER			[IN_DEFAULT]
//
'^^'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^@'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^$'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^('			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^)'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^['			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^]'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^{'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^}'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^;'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^$apr1'		_DONT_CARE			DEFAULT				[IN_DEFAULT]
'^#'			'[A-Fa-f0-9]'		SYMBOL_CODE			[IN_DEFAULT]
'\\\\'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\('			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\)'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\['			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\]'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\{'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
'\\}'			_DONT_CARE			DEFAULT				[IN_DEFAULT]
//
'^switch['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^case['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^for['			_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^taint['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^untaint['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^untaint{'		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^try{'			_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^throw['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^eval('		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^process['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^process{'		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^cache['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^use['			_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]
'^connect['		_DONT_CARE			PARSER_RESERVED		[IN_DEFAULT]

%%handlers
PARSER_VARIABLE		_BETA			_NOT_BETA			_discard_
PARSER_VARIABLE2	_BETA			'}'					_use_
PARSER_FUNCTION		_BETA			_NOT_BETA			_discard_
LOGEXP_COMMENT		'[^\n]'?		'\n'				_discard_
P_COMMENT_BLOCK		'[^\}]'?		'}'					_use_
PARSER_GLOBAL		_BETA			_NOT_BETA			_discard_
P_DEFCOMMENT		'[^\n]'?		'\n'				_discard_
SYMBOL_CODE			'[A-Fa-f0-9]'	'[^A-Fa-f0-9]'		_discard_
ATTR_COM_LINE		'[^\n]'?		'\n'				_discard_

%%tokens
XML_ENTITY			'&'			'[A-Za-z0-9#^]'?		';'			_use_
P_COMMENT_LINE		'\#'		_ORIGINAL_CHAR			'\n'		_discard_
GLOBAL_NUMBER					__STD_NUMBER_OR_FP
FORMAT_STRING		'%'			'[0-9\.]'				'[duoxXf]'	_use_

InState IN_TAG
(
	XML_TAGNAME					__STD_IDENTIFIER
	XML_NUMBER					__STD_NUMBER_OR_FP
	P_COMMENT_LINE	'\#'		_ORIGINAL_CHAR			'\n'		_discard_
)

InState IN_TAGCLOSE
(
	XML_TAGNAME					__STD_IDENTIFIER
	XML_NUMBER					__STD_NUMBER_OR_FP
	P_COMMENT_LINE	'\#'		_ORIGINAL_CHAR			'\n'		_discard_
)

InState IN_DEFXML
(
	XML_DEFNAME					__STD_IDENTIFIER
	XML_DEFNUMBER				__STD_NUMBER_OR_FP
	P_COMMENT_LINE	'\#'		_ORIGINAL_CHAR			'\n'		_discard_
)

InState IN_LOGEXP
(
	LOGEXP_STRING				__STD_HTML_STRING
	GLOBAL_NUMBER				__STD_NUMBER_OR_FP
)

InState IN_DEFINEM
(
	P_DEFNAME		_ALPHA			'[_A-Za-z0-9\-]'?				'[\[]'			_discard_
)

InState IN_TAGATTR
(
	ATTR_STRING		_ATTR_BEGIN		_ATTR_ALLOW?		_ATTR_CLOSE		_discard_
)

InState IN_DEFATTR
(
	ATTR_STRING		_ATTR_BEGIN		_ATTR_ALLOW?		_ATTR_CLOSE		_discard_
)

// State switching definitions
%%states
XML_TAGSTART		(+[IN_TAG] -[IN_TAGATTR] -[IN_DEFATTR])
XML_TAGEND			(-[IN_TAG])

XML_TAGCLOSE		(+[IN_TAGCLOSE] -[IN_TAGATTR] -[IN_DEFATTR])
XML_TAGEND			(-[IN_TAGCLOSE])

XML_DEFSTART		(+[IN_DEFXML] -[IN_TAGATTR] -[IN_DEFATTR])
XML_DEFEND			(-[IN_DEFXML])

TAGATTR_START		(-[IN_TAG] -[IN_DEFXML] +[IN_TAGATTR])
TAGATTR_END			(-[IN_TAGATTR] +[IN_TAG])

DEFATTR_START		(-[IN_TAG] -[IN_DEFXML] +[IN_DEFATTR])
DEFATTR_END			(-[IN_DEFATTR] +[IN_DEFXML])

LOGEXP_START		(+[IN_LOGEXP])
LOGEXP_END			(-[IN_LOGEXP])

DEFCOM_START		(-[IN_DEFPARAM] +[IN_DEFCOM])
DEFCOM_END			(-[IN_DEFCOM] +[IN_DEFAULT])

P_DEFSTART			(-[IN_DEFAULT] -[IN_TAG] -[IN_DEFXML] +[IN_DEFINEM] -[IN_TAGATTR] -[IN_DEFATTR])
P_DEFEND			(-[IN_DEFINEM] -[IN_DEFPARAM] +[IN_DEFAULT])

DPARAM_START		(-[IN_DEFINEM] +[IN_DEFPARAM])
DPARAM_END			(-[IN_DEFPARAM] +[IN_DEFAULT])

%%effects
DEFAULT			[]			DEFAULT_FOREGROUND*		DEFAULT_BACKGROUND*
XML_TAGNAME		[]			clTeal					DEFAULT_BACKGROUND*
XML_DEFNAME		[]			clSpGreen				DEFAULT_BACKGROUND*
XML_ENTITY		[]			DEFAULT_FOREGROUND*		DEFAULT_BACKGROUND*
XML_COMSTART	[fsItalic]	clGray					DEFAULT_BACKGROUND*
PARSER_VARIABLE	[]			clBlood					DEFAULT_BACKGROUND*
PARSER_FUNCTION	[]			clMaroon				DEFAULT_BACKGROUND*
P_COMMENT_LINE	[fsItalic]	clGrayPlus				DEFAULT_BACKGROUND*
PARSER_SERVICE	[fsBold]	clMaroon				DEFAULT_BACKGROUND*
GLOBAL_BRACKET	[]			clCobalt				DEFAULT_BACKGROUND*
PARSER_RESULT	[]			clOrange				DEFAULT_BACKGROUND*
LOGEXP_OPERATOR	[]			clCobalt				DEFAULT_BACKGROUND*
GLOBAL_NUMBER	[]			clFuchsia				DEFAULT_BACKGROUND*
LOGEXP_STRING	[]			clTeal					DEFAULT_BACKGROUND*
TAGATTR_START	[]			clSoftBlue				DEFAULT_BACKGROUND*
FORMAT_STRING	[]			clFuchsia				DEFAULT_BACKGROUND*
P_DEFSTART		[fsBold]	clMaroon				DEFAULT_BACKGROUND*

%%map
DEFAULT				DEFAULT
XML_TAGNAME			XML_TAGNAME
XML_TAGSTART		XML_TAGNAME
XML_TAGEND			XML_TAGNAME
XML_NUMBER			XML_TAGNAME
XML_ENTITY			XML_ENTITY
PARSER_VARIABLE		PARSER_VARIABLE
PARSER_VARIABLE2	PARSER_VARIABLE
PARSER_FUNCTION		PARSER_FUNCTION
P_COMMENT_LINE		P_COMMENT_LINE
P_COMMENT_BLOCK		P_COMMENT_LINE
PARSER_SERVICE		PARSER_SERVICE
GLOBAL_BRACKET		GLOBAL_BRACKET
PARSER_RESERVED		GLOBAL_BRACKET
PARSER_GLOBAL		GLOBAL_BRACKET
PARSER_RESULT		PARSER_RESULT
LOGEXP_START		GLOBAL_BRACKET
LOGEXP_END			GLOBAL_BRACKET
LOGEXP_OPERATOR		LOGEXP_OPERATOR
LOGEXP_COMMENT		P_COMMENT_LINE
XML_COMSTART		XML_COMSTART
XML_COMEND			XML_COMSTART
P_DEFSTART			P_DEFSTART
P_DEFEND			P_DEFSTART
P_DEFNAME			P_DEFSTART
P_DEFCOMMENT		P_COMMENT_LINE
P_DELIMITER			GLOBAL_BRACKET
GLOBAL_NUMBER		GLOBAL_NUMBER
FORMAT_STRING		FORMAT_STRING
XML_DEFNAME			XML_DEFNAME
XML_DEFSTART		XML_DEFNAME
XML_DEFEND			XML_DEFNAME
XML_DEFNUMBER		XML_DEFNAME
SYMBOL_CODE			DEFAULT
XML_TAGCLOSE		XML_TAGNAME
LOGEXP_STRING		LOGEXP_STRING
DPARAM_START		GLOBAL_BRACKET
DPARAM_END			GLOBAL_BRACKET
DPARAM_SEP			GLOBAL_BRACKET
DEFCOM_START		GLOBAL_BRACKET
DEFCOM_END			GLOBAL_BRACKET
TAGATTR_START		TAGATTR_START
TAGATTR_END			TAGATTR_START
DEFATTR_START		TAGATTR_START
DEFATTR_END			TAGATTR_START
ATTR_STRING			TAGATTR_START
TAGNAME_MISC		TAGATTR_START
NAMESPACE_SEP		TAGATTR_START
ATTR_COM_LINE		P_COMMENT_LINE